************************************************************************
*****                                                              *****
***** @segment name: xs_ioDS.f                                     *****
*****                                                              *****
***** @task: read the dynamical system                             *****
*****                                                              *****
***** @see: header of xGMshow.f                                    *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************

***********************************************************************
* === initialize dynamics =========================================== *
      subroutine initDS
      implicit none
      include 'xv_ds.include'
      integer   iEM, jEM, kEM, iVF
      real*8    fAi(-NAMP:NEMP)
      real*8    fBi(-NAMP:NVFP)
* --- begin of subroutine ------------------------------------------- *
* --- initialize natural dynamics
      do iEM = 1, NEMP
        do jEM = -NAMP, NEMP
        do kEM = -NAMP, NEMP
          fQijk(iEM,jEM,kEM) = 0.0
        end do
        end do
        do iVF = -NAMP, NVFP
          fGij(iEM,iVF) = 0.0
        end do
      end do

* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === initialize the state ========================================== *
      subroutine initState(fAi,fBi)
      implicit none
      include 'xv_ds.include'
      integer   iEM, iVF
      real*8    fAi(-NAMP:NEMP)
      real*8    fBi(-NAMP:NVFP)
* --- begin of subroutine ------------------------------------------- *
* --- initialize state
      do iEM = -NAMP, NEMP
         fAi(iEM) = 0.0
      end do
      fAi(0) = 1.0

* --- initialize control
      do iVF = -NAMP, NVFP
         fBi(iVF) = 0.0
      end do
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === read dynamicsl system ========================================= *
      subroutine readDS(fAi)
      implicit none
      include 'xv_ds.include'
      include 'xv_ic.include'

      real*8  fAi(-NAMP:NEMP)

      integer iEM, jEM, kEM
      real*8  fVal

      integer iChar
      character line*120
      character FileName*120
* --- begin of subroutine ------------------------------------------- *
      write(*,*)
      write(*,*) '>>> Reading yDS.dat ...'
* --- open job file ------------------------------------------------- *
      open(10,file='yDS.dat',status='old',form='formatted')
* --- read job file
* --- skip comments
  101 read(10,'(a)',end=999) line
      if (line(1:1).eq.'[') then
      write(*,'(a)') line
        goto 101
      end if
* --- read dimensions
      read(line,*) NVF,NAM,NEM
      write(*,*) 'Dimensions: ',  NVF, NAM, NEM

* --- read file for dynamical systems ------------------------------- *
* --- read file name with dynamics
      read(10,'(a)',end=999) FileName
      iChar = index(FileName,'.dat') + 3 
      write(*,*) '>>>>>> Reading ',FileName(1:iChar),'...'
      open(11,file=FileName(1:iChar),status='old',form='formatted')

* --- read file and skip comments
  201 read(11,'(a)',end=299) line
      if (line(1:1).eq.'[') goto 201
      read(line,*), iEM, jEM, kEM, fQijk(iEM, jEM, kEM)
      write(*,'(3(a,i4),a,f12.5)') 
     &        'q(',iEM,';',jEM,';',kEM,')=',fQijk(iEM,jEM,kEM)
      goto 201

* --- close file with natural dynamics
  299 write(*,*) '>>>>>> Closing ',FileName(1:iChar),'...'
      close(11)

* --- open file with forcing term ----------------------------------- *
      if (NVF.gt.0) then
* ----- read file name with dynamics
        read(10,'(a)',end=999) FileName
        iChar = index(FileName,'.dat') + 3 
        write(*,*) '>>>>>> Reading ',FileName(1:iChar),'...'
        open(11,file=FileName(1:iChar),status='old',form='formatted')

* ----- read file and skip comments
  301   read(11,'(a)',end=399) line
        if (line(1:1).eq.'[') goto 301
        read(line,*), iEM, jEM,  fGij(iEM, jEM)
        write(*,'(2(a,i4),a,f12.5)') 
     &          'g(',iEM,';',jEM,')=',fGij(iEM,jEM)
        goto 301

* ----- close file with natural dynamics
  399   write(*,*) '>>>>>> Closing ',FileName(1:iChar),'...'
        close(11)
      end if

* --- initial condition --------------------------------------------- *
      do iEM = -NAMP,NEMP
        fAi(iEM) = 0.0
      end do
* --- read file name with initial conditions
      read(10,'(a)',end=999) FileName
      iChar = index(FileName,'.dat') +3 
      write(*,*) '>>>>>> Reading ',FileName(1:iChar),'...'
      open(11,file=FileName(1:iChar),status='old',form='formatted')
* --- read file and skip comments
  401 read(11,'(a)',end=499) line
      if (line(1:1).eq.'[') then
        write(*,'(a)') line
        goto 401
      end if

      read(line,*)  t_Begin
      write(*,'(a,f12.5)') 't_0  =',t_Begin

      read(11,*)    t_End
      write(*,'(a,f12.5)') 't_1  =',t_End

      read(11,*)    t_Step
      write(*,'(a,f12.5)') 'dt   =',t_Step
      read(11,*)    t_Diff
      write(*,'(a,f12.5)') 'dt_L =',t_Diff

  411 read(11,*,end=499) iEM, fVal
      fAi(iEM) = fVal
      write(*,'(1(a,i4),a,f12.5)') 
     &        'a(',iEM,')=',fAi(iEM)
      goto 411
* --- close file with initial conditions
  499 write(*,*) '>>>>>> Closing ',FileName(1:iChar),'...'
      close(11)

* --- close job file ------------------------------------------------ *
  999  continue
      close(10)
* --- end of subroutine --------------------------------------------- *
      return
      end